<?php
namespace app\common;
use think\Paginator;
class CustomPaginate extends Paginator
{
 
    /**
     * 上一页按钮
     * @param string $text
     * @return string|null
     */
    protected function getPreviousButton(string $text = "上一页"): ?string
    {
 
        if ($this->currentPage() <= 1) {
            return null;
        }
 
        $url = $this->url(
            $this->currentPage() - 1
        );
 
        return $this->getPageLinkWrapper($url, $text);
    }
 
    /**
     * 下一页按钮
     * @param string $text
     * @return string|null
     */
    protected function getNextButton(string $text = '下一页'): ?string
    {
        if (!$this->hasMore) {
            return null;
        }
 
        $url = $this->url($this->currentPage() + 1);
 
        return $this->getPageLinkWrapper($url, $text);
    }
 
    /**
     * 页码按钮
     * @return string
     */
    protected function getLinks(): string
    {
        if ($this->simple) {
            return '';
        }
 
        $block = [
            'first'  => null,
            'slider' => null,
            'last'   => null,
        ];
 
        $side   = 3;
        $window = $side * 2;
 
        if ($this->lastPage < $window + 6) {
            $block['first'] = $this->getUrlRange(1, $this->lastPage);
        } elseif ($this->currentPage <= $window) {
            $block['first'] = $this->getUrlRange(1, $window + 2);
            $block['last']  = $this->getUrlRange($this->lastPage - 1, $this->lastPage);
        } elseif ($this->currentPage > ($this->lastPage - $window)) {
            $block['first'] = $this->getUrlRange(1, 2);
            $block['last']  = $this->getUrlRange($this->lastPage - ($window + 2), $this->lastPage);
        } else {
            $block['first']  = $this->getUrlRange(1, 2);
            $block['slider'] = $this->getUrlRange($this->currentPage - $side, $this->currentPage + $side);
            $block['last']   = $this->getUrlRange($this->lastPage - 1, $this->lastPage);
        }
 
        $html = '';
 
        if (is_array($block['first'])) {
            $html .= $this->getUrlLinks($block['first']);
        }
 
        if (is_array($block['slider'])) {
            $html .= $this->getDots();
            $html .= $this->getUrlLinks($block['slider']);
        }
 
        if (is_array($block['last'])) {
            $html .= $this->getDots();
            $html .= $this->getUrlLinks($block['last']);
        }
 
        return $html;
    }
 
    /**
     * 渲染分页html
     * @return mixed
     */
    public function render()
    {
        if ($this->hasPages()) {
            if ($this->simple) {
                return sprintf(
                    '<br><div class="layui-form-item"><div class="layui-inline"><div class="layui-btn-group">%s %s</div></div></div>',
                    $this->getPreviousButton(),
                    $this->getNextButton()
                );
            } else {
                return sprintf(
                    '<br><div class="layui-form-item"><div class="layui-inline">%s<div class="layui-btn-group">%s %s %s %s %s</div></div></div>',
                    $this->getTotal(),
                    $this->getfirst(),
                    $this->getPreviousButton(),
                    $this->getLinks(),
                    $this->getNextButton(),
                    $this->getlast()
                );
            }
        }
    }
    // 首页
    protected function getfirst($text="首页"){
        if ($this->currentPage() <= 1) {
            return null;
        }
        $url = $this->url(1);
        return $this->getPageLinkWrapper($url, $text);
    }
    // 尾页
    protected function getlast($text="尾页"){
        if ($this->currentPage() == $this->lastPage) {
            return null;
        }
        $url = $this->url($this->lastPage);
        return $this->getPageLinkWrapper($url, $text);
    }
    // 总页码
    protected function getTotal(){
        return '<span class="layui-btn layui-btn-primary" style="margin-right: 10px;">第'.$this->currentPage.'页,共'.$this->lastPage.'页/'.$this->total().'条数据</span>';
    }
    /**
     * 生成一个可点击的按钮
     *
     * @param  string $url
     * @param  string $page
     * @return string
     */
    protected function getAvailablePageWrapper(string $url, string $page): string
    {
        return '<a href="' . htmlentities($url) . '" class="layui-btn">' . $page . '</a>';
    }
 
    /**
     * 生成一个禁用的按钮
     *
     * @param  string $text
     * @return string
     */
    protected function getDisabledTextWrapper(string $text): string
    {
        return '<span disabled class="layui-btn layui-btn-danger">' . $text . '</span>';
    }
 
    /**
     * 生成一个激活的按钮
     *
     * @param  string $text
     * @return string
     */
    protected function getActivePageWrapper(string $text): string
    {
        return '<span class="layui-btn layui-btn-danger">' . $text . '</span>';
    }
 
    /**
     * 生成省略号按钮
     *
     * @return string
     */
    protected function getDots(): string
    {
        return $this->getDisabledTextWrapper('...');
    }
 
    /**
     * 批量生成页码按钮.
     *
     * @param  array $urls
     * @return string
     */
    protected function getUrlLinks(array $urls): string
    {
        $html = '';
 
        foreach ($urls as $page => $url) {
            $html .= $this->getPageLinkWrapper($url, $page);
        }
 
        return $html;
    }
 
    /**
     * 生成普通页码按钮
     *
     * @param  string $url
     * @param  string    $page
     * @return string
     */
    protected function getPageLinkWrapper(string $url, string $page): string
    {
        if ($this->currentPage() == $page) {
            return $this->getActivePageWrapper($page);
        }
        return $this->getAvailablePageWrapper($url, $page);
    }
 
}